
use strict;
use warnings;

no warnings 'redefine';

use Test::More tests => 33;

use PerlMongers::Common;
use PerlMongers::DataSet;
use PerlMongers::Period;

use Data::Dumper;
use File::Slurp;

sub copyAndRemoveKey
{
	my ($key, @data) = @_;
	
	my @output;

	my $len = @data;
	for (my $i = 0; $i < $len; ++$i)
	{
		my $base_ref = $data [$i];
		my $x_base_ref = PerlMongers::Common::copyHashRef ($base_ref);

		delete $x_base_ref->{$key};

		push (@output, $x_base_ref);
	}

	return @output;
}

my $data_set_name = "EURAUD_daily";
my @base_data = PerlMongers::DataSet::loadDataSet ($data_set_name);
my @compare_base_data = copyAndRemoveKey ("OHLCDataSetID", @base_data);

{
	local $PerlMongers::DataSet::OHLCDataSets = "tOHLCDataSets";
	local $PerlMongers::DataSet::OHLCData = "tOHLCData";

	PerlMongers::DataSet::_createTempTables ();

	my $currencyPair = "XYZZYX";
	my $dataSetName = $currencyPair . "_daily";

	my $dataset_id = PerlMongers::DataSet::createDataSet ($dataSetName, $currencyPair, 1440, @base_data);

	my @copy_data = PerlMongers::DataSet::loadDataSet ($dataSetName);

	is (@copy_data, @base_data, "Identical number of points");

	# Unfortunately, OHLCDataSetID will be different between the 2 runs, so I
	# have to remove it before comparing the datasets, quickly

	my @compare_copy_data = copyAndRemoveKey ("OHLCDataSetID", @base_data);

	is_deeply (\@compare_copy_data, \@compare_base_data, "data created identically");

	my $copy_dataset_id = PerlMongers::DataSet::getDataSetID ($dataSetName);

	is ($copy_dataset_id, $dataset_id, "getDataSetID is correct");

	PerlMongers::DataSet::deleteDataSet ($dataSetName);

	my $dbh = PerlMongers::DB::getPgDBH ();
	my $sql = qq~SELECT COUNT(*) FROM $PerlMongers::DataSet::OHLCDataSets WHERE "OHLCDataSetID" = ?~;
	my $sth = $dbh->prepare ($sql);
	my $results = $sth->execute ($dataset_id);

	my $ref = $sth->fetchrow_arrayref ();
	my $count = $ref->[0];

	is ($count, 0, 'deleteDataSet deleted OHLCDataSet');

	$sql = qq~SELECT COUNT(*) FROM $PerlMongers::DataSet::OHLCData  WHERE "OHLCDataSetID" = ?~;
	$sth = $dbh->prepare ($sql);
	$results = $sth->execute ($dataset_id);

	$ref = $sth->fetchrow_arrayref ();
	$count = $ref->[0];

	is ($count, 0, 'deleteDataSet deleted OHLCData');

	$dataset_id = PerlMongers::DataSet::createDataSet ($dataSetName, $currencyPair, 1440, @base_data);

	@copy_data = PerlMongers::DataSet::loadDataSet ($dataSetName);
	@compare_copy_data = copyAndRemoveKey ("OHLCDataSetID", @base_data);

	is_deeply (\@compare_copy_data, \@compare_base_data, "data re-created identically");

	my @datasets = PerlMongers::DataSet::getAllDataSets ();
	
	is (@datasets, 1, "getAllDataSets returned the correct number");

	my $compare_ref = {
		'Pair' => $currencyPair,
		'period' => 1440,
		'Name' => $dataSetName,
		'OHLCDataSetID' => $dataset_id,
	};

	is_deeply ($datasets [0], $compare_ref, 'dataset is correct');

	my @alldates = PerlMongers::DataSet::getAllDates ($dataSetName);

	is (@alldates, @base_data, "getAllDates returns the correct number of dates");

	my @base_dates;

	foreach my $ref (@base_data)
	{
		my $date = PerlMongers::Common::trim ($ref->{'Date'});
		$date = substr ($date, 0, 10);
	
		push (@base_dates, $date);	
	}

	is_deeply (\@alldates, \@base_dates, "getAllDates returns correct dates");

	my $youngest_date = PerlMongers::DataSet::_getYoungestDate ($dataset_id);

	is ($youngest_date, $base_data [-1]->{"Date"}, "Youngest Date is correct");

	my $youngest_epoch_time = PerlMongers::Period::getTimeFromDate ($youngest_date);

	my $base_ref = $base_data [-1];

	my @append_data;

	my $xwday;
	my $epoch_time = $youngest_epoch_time;

	foreach (1..10)
	{
		$epoch_time += 86400 + (3 * 3600);

		($epoch_time, $xwday) = PerlMongers::Period::findMidnightPlusOne ($epoch_time);

		my $ref = PerlMongers::Common::copyHashRef ($base_ref);

		$ref->{'Date'} = sprintf ("%-20.20s", PerlMongers::Period::convertTimeToDate ($epoch_time));
		$ref->{'EpochTime'} = $epoch_time;

		push (@append_data, $ref);
	}

	my @new_base_data;
	push (@new_base_data, @base_data, @append_data);
	my @compare_new_base_data = copyAndRemoveKey ("OHLCDataSetID", @new_base_data);

	PerlMongers::DataSet::updateDataSet ($dataSetName, $currencyPair, 1440, @append_data);

	my @after_append = PerlMongers::DataSet::loadDataSet ($dataSetName);
	my @compare_after_append = copyAndRemoveKey ("OHLCDataSetID", @after_append);

	is (@after_append, @new_base_data, "updateDataSet same number of points");
	is_deeply (\@compare_after_append, \@compare_new_base_data, "updateDataSet is working");

	$ref = PerlMongers::DataSet::getDataSetRecord ($dataSetName);

	is_deeply ($ref, $compare_ref, "getDataSetRecord works correctly");

	@after_append = PerlMongers::DataSet::loadDataSet ($dataSetName, undef, undef, 1);

	$ref = {
		"Open" => 1,
		"High" => 2,
		"Low" => 3,
		"Close" => 4,
		"ohlc_id" => $after_append [-1]->{'ohlc_id'},
	};

	PerlMongers::DataSet::updateCandle ($ref);

	my @after_update = PerlMongers::DataSet::loadDataSet ($dataSetName, undef, undef, 1);
	my $yref = $after_append [-1];

	$yref->{'Open'} = $ref->{'Open'};
	$yref->{'High'} = $ref->{'High'};
	$yref->{'Low'} = $ref->{'Low'};
	$yref->{'Close'} = $ref->{'Close'};

	my $xref = $after_update [-1];

	is_deeply ($xref, $yref, "updateCandle works");

	# pick a candle in the middle
	
	my $idx = int ($#after_append / 2);

	# because of the origin of the data, we cannot compare on all keys,
	# so I remove the ones that only come from one source or have no
	# chance of being the same.

	sub cleanCandleRef
	{
		my ($ref)= @_;

		delete $ref->{'OHLCDataSetID'} if exists $ref->{'OHLCDataSetID'};
		delete $ref->{'order'} if exists $ref->{'order'};
		delete $ref->{'rows'} if exists $ref->{'rows'};
		delete $ref->{'Error'} if exists $ref->{'Error'};

		return $ref;
	}

	my $ccandle_ref = cleanCandleRef ($after_update [$idx]);
	my $cref = cleanCandleRef (PerlMongers::DataSet::getCandle ($dataSetName, PerlMongers::Common::trim ($ccandle_ref->{'Date'})));

	is_deeply ($cref, $ccandle_ref, "getCandle works correctly");

	my @carray = PerlMongers::DataSet::getCandleFloor ($dataSetName, PerlMongers::Common::trim ($ccandle_ref->{'Date'}));

	is (@carray, 1, "getCandleFloor returned 1 as it should");

	$cref = $carray [0];
	$cref = cleanCandleRef ($cref);
	
	ok ($cref->{'Date'} eq $ccandle_ref->{'Date'}, "Candle Floor is equal to candle");

	@carray = PerlMongers::DataSet::getCandleFloor ($dataSetName, PerlMongers::Common::trim ($ccandle_ref->{'Date'}), 1);

	is (@carray, 1, "getCandleFloor returned 1 as it should");

	$cref = $carray [0];
	$cref = cleanCandleRef ($cref);
	
	ok ($cref->{'Date'} eq $ccandle_ref->{'Date'}, "Candle Floor is equal to candle");

	@carray = PerlMongers::DataSet::getCandleFloor ($dataSetName, PerlMongers::Common::trim ($ccandle_ref->{'Date'}), 3);

	is (@carray, 3, "getCandleFloor returned 3 as it should");

	$cref = $carray [2];
	$cref = cleanCandleRef ($cref);
	
	ok ($cref->{'Date'} lt $ccandle_ref->{'Date'}, "Candle Floor is less than candle");

	$cref = $carray [1];
	$cref = cleanCandleRef ($cref);
	
	ok ($cref->{'Date'} lt $ccandle_ref->{'Date'}, "Candle Floor is less than candle");

	$cref = $carray [0];
	$cref = cleanCandleRef ($cref);
	
	ok ($cref->{'Date'} eq $ccandle_ref->{'Date'}, "Candle Floor is equal to candle");

	my $start_candle = $after_update [$idx - 5];
	my $end_candle   = $after_update [$idx];

	@carray = PerlMongers::DataSet::getCandles ($dataSetName, $start_candle->{'Date'}, $end_candle->{'Date'});

	is (@carray, 6, "getCandles returned 6 as it should");

	my $start = $idx - 5;
	for (my $i = $start; $i <= $idx; ++$i)
	{
		$cref = $carray [$i - $start];
		$cref = cleanCandleRef ($cref);

		$ccandle_ref = cleanCandleRef ($after_update [$i]);

		is_deeply ($cref, $ccandle_ref, "Record (" . ($i - $start) . ") same");
	}

	@carray = PerlMongers::DataSet::getAllCandles ($dataSetName);
	is (@carray, @after_append, "getAllCandles returns correct number");

	my $baseTime = PerlMongers::DataSet::getBaseTime ($dataSetName);

	is ($baseTime, 86400, "BaseTime is correct");

	$cref = PerlMongers::DataSet::getLastCandleOfDay ($dataSetName, $ccandle_ref->{'Date'});
	$cref = cleanCandleRef ($cref);

	is_deeply ($cref, $ccandle_ref, "getLastCandleOfDay is correct");
}

1;

